"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var SteppableInput_1 = require("../SteppableInput/SteppableInput");
var Control_1 = require("../Control/Control");
var withTheme_1 = require("../../utils/withTheme");
var Input_1 = require("../input/Input");
var with_defaults_1 = require("../../utils/with-defaults");
var Option_1 = require("fp-ts/lib/Option");
var pipeable_1 = require("fp-ts/lib/pipeable");
exports.NUMERIC_STEPPER = Symbol('NumericStepper');
var RawNumericStepper = /** @class */ (function (_super) {
    tslib_1.__extends(RawNumericStepper, _super);
    function RawNumericStepper(props) {
        var _this = _super.call(this, props) || this;
        _this.onKeyDown = function (event) {
            switch (event.keyCode) {
                case Control_1.KeyCode.Up:
                    _this.increase();
                    break;
                case Control_1.KeyCode.Down:
                    _this.decrease();
                    break;
            }
        };
        _this.onInputChange = function (value) {
            if (!_this.props.manualEdit) {
                return;
            }
            if (_this.state.isFocused) {
                var onValueChange = _this.props.onValueChange;
                var newValue = _this.parseValue(pipeable_1.pipe(Option_1.fromNullable(value), Option_1.getOrElse(function () { return ''; })));
                _this.setState({
                    displayedValue: _this.formatValue(newValue),
                });
                onValueChange && onValueChange(newValue);
            }
        };
        _this.onFocus = function () {
            _this.setState({
                isFocused: true,
            });
        };
        _this.onBlur = function (event) {
            var _a = _this.props, manualEdit = _a.manualEdit, onValueChange = _a.onValueChange;
            if (!manualEdit) {
                return;
            }
            var value = event.target.value;
            var newValue = _this.parseValue(value);
            _this.setState({
                isFocused: false,
                displayedValue: _this.formatValue(newValue),
            });
            onValueChange && onValueChange(newValue);
        };
        _this.onWheel = function (event) {
            var isDisabled = _this.props.isDisabled;
            var isFocused = _this.state.isFocused;
            if (!isDisabled && isFocused) {
                if (event.deltaY < 0) {
                    _this.increase();
                }
                else {
                    _this.decrease();
                }
            }
        };
        _this.decrease = function () {
            _this.step(-1);
        };
        _this.increase = function () {
            _this.step(1);
        };
        _this.onIncrement = function () {
            _this.increase();
        };
        _this.onDecrement = function () {
            _this.decrease();
        };
        var state = {
            isFocused: false,
            displayedValue: props.value,
        };
        if (props.value) {
            state.displayedValue = _this.formatValue(props.value);
        }
        _this.state = state;
        return _this;
    }
    RawNumericStepper.prototype.componentWillReceiveProps = function (newProps) {
        this.setState({
            displayedValue: this.formatValue(newProps.value),
        });
    };
    RawNumericStepper.prototype.render = function () {
        var displayedValue = this.state.displayedValue;
        var _a = this.props, theme = _a.theme, decrementIcon = _a.decrementIcon, incrementIcon = _a.incrementIcon, value = _a.value, isDisabled = _a.isDisabled, clearIcon = _a.clearIcon, min = _a.min, max = _a.max, SteppableInput = _a.SteppableInput;
        var hasError = isNaN(value) || value < min || value > max;
        var isIncrementButtonDisabled = value >= max;
        var isDecrementButtonDisabled = value <= min;
        return (React.createElement(SteppableInput, { isDisabled: isDisabled, isDecrementButtonDisabled: isDecrementButtonDisabled, isIncrementButtonDisabled: isIncrementButtonDisabled, error: hasError, theme: theme.SteppableInput, onDecrement: this.onDecrement, onIncrement: this.onIncrement, decrementIcon: decrementIcon, incrementIcon: incrementIcon, clearIcon: clearIcon },
            React.createElement(Input_1.Input, { type: "text", onKeyDown: this.onKeyDown, onFocus: this.onFocus, onWheel: this.onWheel, tabIndex: -1, error: hasError, theme: theme.Input, isDisabled: isDisabled, value: "" + displayedValue, onBlur: this.onBlur, onValueChange: this.onInputChange })));
    };
    RawNumericStepper.prototype.getPrecision = function (step) {
        if (Number.isInteger(step)) {
            return 0;
        }
        else {
            return step.toString(10).split('.')[1].length;
        }
    };
    RawNumericStepper.prototype.step = function (n) {
        var _a = this.props, step = _a.step, min = _a.min, max = _a.max, onValueChange = _a.onValueChange;
        var value = this.props.value;
        var num = value + n * step;
        var precision = this.getPrecision(step);
        var frac = Math.pow(10, precision);
        var newValue = Math.round(num * frac) / frac;
        if (newValue >= min && newValue <= max && newValue !== value) {
            onValueChange && onValueChange(newValue);
        }
    };
    RawNumericStepper.prototype.formatValue = function (value) {
        var formatter = this.props.formatter;
        return formatter ? formatter(value) : value;
    };
    /**
     * Trying to parse the entered string to a number.
     * Done here because `onChange` should return a valid numeric value if possible,
     * e.g. for proper interop with `stateful`.
     */
    RawNumericStepper.prototype.parseValue = function (value) {
        var parser = this.props.parser;
        if (parser) {
            return parser(value);
        }
        else {
            var num = Number(value);
            return isNaN(num) ? value : num;
        }
    };
    RawNumericStepper.propTypes = {
        value: function (props) {
            var value = props.value, min = props.min, max = props.max;
            var type = typeof value;
            if (type === 'number' && (value < min || value > max)) {
                throw new Error("Value: " + value + " should be greater than min: " + min + " and lower than max: " + max);
            }
        },
        min: function (props) {
            var min = props.min, max = props.max;
            var type = typeof min;
            if (type !== 'undefined') {
                if (type !== 'number') {
                    throw new Error('Min should be a number');
                }
                else {
                    if (min >= max) {
                        throw new Error('Invalid min value. Min value should be lower than max');
                    }
                }
            }
        },
        max: function (props) {
            var min = props.min, max = props.max;
            var type = typeof max;
            if (type !== 'undefined') {
                if (type !== 'number') {
                    throw new Error('Max should be a string or a number');
                }
                else {
                    if (max <= min) {
                        throw new Error('Invalid Max value. Max value should be greater than min');
                    }
                }
            }
        },
    };
    return RawNumericStepper;
}(React.Component));
exports.NumericStepper = withTheme_1.withTheme(exports.NUMERIC_STEPPER)(with_defaults_1.withDefaults({
    step: 1,
    manualEdit: true,
    min: Number.NEGATIVE_INFINITY,
    max: Number.POSITIVE_INFINITY,
    SteppableInput: SteppableInput_1.SteppableInput,
})(RawNumericStepper));
//# sourceMappingURL=NumericStepper.js.map